@@ -16,13 +16,15 @@ import argparse
 import copy
 import os
 
+import extras
 import fixtures
 import testtools
 import yaml
 
+from openstack.config import defaults
+
 from os_client_config import cloud_config
 from os_client_config import config
-from os_client_config import defaults
 from os_client_config import exceptions
 from os_client_config.tests import base
 
@@ -45,7 +47,7 @@ class TestConfig(base.TestCase):
             cloud for cloud in base.USER_CONF['clouds'].keys()
         ] + ['_test_cloud_regions']
         configured_clouds = [cloud.name for cloud in clouds]
-        self.assertItemsEqual(user_clouds, configured_clouds)
+        self.assertCountEqual(user_clouds, configured_clouds)
 
     def test_get_one_cloud(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
@@ -54,6 +56,25 @@ class TestConfig(base.TestCase):
         self.assertIsInstance(cloud, cloud_config.CloudConfig)
         self.assertEqual(cloud.name, '')
 
+    def test_get_one_cloud_default_cloud_from_file(self):
+        single_conf = base._write_yaml({
+            'clouds': {
+                'single': {
+                    'auth': {
+                        'auth_url': 'http://example.com/v2',
+                        'username': 'testuser',
+                        'password': 'testpass',
+                        'project_name': 'testproject',
+                    },
+                    'region_name': 'test-region',
+                }
+            }
+        })
+        c = config.OpenStackConfig(config_files=[single_conf],
+                                   vendor_files=[self.vendor_yaml])
+        cc = c.get_one_cloud()
+        self.assertEqual(cc.name, 'single')
+
     def test_get_one_cloud_auth_defaults(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml])
         cc = c.get_one_cloud(cloud='_test-cloud_', auth={'username': 'user'})
@@ -204,6 +225,7 @@ class TestConfig(base.TestCase):
         self.assertEqual(
             ['a-private', 'another-private', 'split-no-default'],
             cc.get_internal_networks())
+        self.assertEqual('a-public', cc.get_nat_source())
         self.assertEqual('another-private', cc.get_nat_destination())
         self.assertEqual('another-public', cc.get_default_network())
         self.assertEqual(
@@ -219,6 +241,7 @@ class TestConfig(base.TestCase):
         cc = c.get_one_cloud('_test-cloud-domain-scoped_')
         self.assertEqual([], cc.get_external_networks())
         self.assertEqual([], cc.get_internal_networks())
+        self.assertIsNone(cc.get_nat_source())
         self.assertIsNone(cc.get_nat_destination())
         self.assertIsNone(cc.get_default_network())
 
@@ -226,7 +249,7 @@ class TestConfig(base.TestCase):
         c = config.OpenStackConfig(config_files=['nonexistent'],
                                    vendor_files=['nonexistent'],
                                    secure_files=[self.secure_yaml])
-        cc = c.get_one_cloud(cloud='_test_cloud_no_vendor')
+        cc = c.get_one_cloud(cloud='_test_cloud_no_vendor', validate=False)
         self.assertEqual('testpass', cc.auth['password'])
 
     def test_get_cloud_names(self):
@@ -354,6 +377,66 @@ class TestConfig(base.TestCase):
         self.assertEqual(region, {'name': 'no-cloud-region', 'values': {}})
 
 
+class TestExcludedFormattedConfigValue(base.TestCase):
+    # verify LaunchPad bug #1635696
+    #
+    # get_one_cloud() and get_one_cloud_osc() iterate over config
+    # values and try to expand any variables in those values by
+    # calling value.format(), however some config values
+    # (e.g. password) should never have format() applied to them, not
+    # only might that change the password but it will also cause the
+    # format() function to raise an exception if it can not parse the
+    # format string. Examples would be single brace (e.g. 'foo{')
+    # which raises an ValueError because it's looking for a matching
+    # end brace or a brace pair with a key value that cannot be found
+    # (e.g. 'foo{bar}') which raises a KeyError.
+
+    def setUp(self):
+        super(TestExcludedFormattedConfigValue, self).setUp()
+
+        self.args = dict(
+            auth_url='http://example.com/v2',
+            username='user',
+            project_name='project',
+            region_name='region2',
+            snack_type='cookie',
+            os_auth_token='no-good-things',
+        )
+
+        self.options = argparse.Namespace(**self.args)
+
+    def test_get_one_cloud_password_brace(self):
+        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
+                                   vendor_files=[self.vendor_yaml])
+
+        password = 'foo{'       # Would raise ValueError, single brace
+        self.options.password = password
+        cc = c.get_one_cloud(
+            cloud='_test_cloud_regions', argparse=self.options, validate=False)
+        self.assertEqual(cc.password, password)
+
+        password = 'foo{bar}'   # Would raise KeyError, 'bar' not found
+        self.options.password = password
+        cc = c.get_one_cloud(
+            cloud='_test_cloud_regions', argparse=self.options, validate=False)
+        self.assertEqual(cc.password, password)
+
+    def test_get_one_cloud_osc_password_brace(self):
+        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
+                                   vendor_files=[self.vendor_yaml])
+        password = 'foo{'       # Would raise ValueError, single brace
+        self.options.password = password
+        cc = c.get_one_cloud_osc(
+            cloud='_test_cloud_regions', argparse=self.options, validate=False)
+        self.assertEqual(cc.password, password)
+
+        password = 'foo{bar}'   # Would raise KeyError, 'bar' not found
+        self.options.password = password
+        cc = c.get_one_cloud_osc(
+            cloud='_test_cloud_regions', argparse=self.options, validate=False)
+        self.assertEqual(cc.password, password)
+
+
 class TestConfigArgparse(base.TestCase):
 
     def setUp(self):
@@ -384,7 +467,7 @@ class TestConfigArgparse(base.TestCase):
                                    vendor_files=[self.vendor_yaml])
 
         cc = c.get_one_cloud(
-            cloud='_test_cloud_regions', argparse=self.options)
+            cloud='_test_cloud_regions', argparse=self.options, validate=False)
         self.assertEqual(cc.region_name, 'region2')
         self.assertEqual(cc.snack_type, 'cookie')
 
@@ -481,7 +564,7 @@ class TestConfigArgparse(base.TestCase):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
                                    vendor_files=[self.vendor_yaml])
 
-        cc = c.get_one_cloud(argparse=self.options)
+        cc = c.get_one_cloud(argparse=self.options, validate=False)
         self.assertIsNone(cc.cloud)
         self.assertEqual(cc.region_name, 'region2')
         self.assertEqual(cc.snack_type, 'cookie')
@@ -490,7 +573,7 @@ class TestConfigArgparse(base.TestCase):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
                                    vendor_files=[self.vendor_yaml])
 
-        cc = c.get_one_cloud(**self.args)
+        cc = c.get_one_cloud(validate=False, **self.args)
         self.assertIsNone(cc.cloud)
         self.assertEqual(cc.region_name, 'region2')
         self.assertEqual(cc.snack_type, 'cookie')
@@ -577,6 +660,37 @@ class TestConfigArgparse(base.TestCase):
         self.assertEqual(cc.region_name, 'region2')
         self.assertEqual('my-network', cc.config['external_network'])
 
+    def test_get_one_cloud_no_yaml_no_cloud(self):
+        c = config.OpenStackConfig(load_yaml_config=False)
+
+        self.assertRaises(
+            exceptions.OpenStackConfigException,
+            c.get_one_cloud,
+            cloud='_test_cloud_regions', region_name='region2', argparse=None)
+
+    def test_get_one_cloud_no_yaml(self):
+        c = config.OpenStackConfig(load_yaml_config=False)
+
+        cc = c.get_one_cloud(
+            region_name='region2', argparse=None,
+            **base.USER_CONF['clouds']['_test_cloud_regions'])
+        # Not using assert_cloud_details because of cache settings which
+        # are not present without the file
+        self.assertIsInstance(cc, cloud_config.CloudConfig)
+        self.assertTrue(extras.safe_hasattr(cc, 'auth'))
+        self.assertIsInstance(cc.auth, dict)
+        self.assertIsNone(cc.cloud)
+        self.assertIn('username', cc.auth)
+        self.assertEqual('testuser', cc.auth['username'])
+        self.assertEqual('testpass', cc.auth['password'])
+        self.assertFalse(cc.config['image_api_use_tasks'])
+        self.assertTrue('project_name' in cc.auth or 'project_id' in cc.auth)
+        if 'project_name' in cc.auth:
+            self.assertEqual('testproject', cc.auth['project_name'])
+        elif 'project_id' in cc.auth:
+            self.assertEqual('testproject', cc.auth['project_id'])
+        self.assertEqual(cc.region_name, 'region2')
+
     def test_fix_env_args(self):
         c = config.OpenStackConfig(config_files=[self.cloud_yaml],
                                    vendor_files=[self.vendor_yaml])
@@ -622,7 +736,7 @@ class TestConfigArgparse(base.TestCase):
                                    vendor_files=[self.vendor_yaml])
 
         cc = c.get_one_cloud(
-            cloud='envvars', argparse=self.options)
+            cloud='envvars', argparse=self.options, validate=False)
         self.assertEqual(cc.auth['project_name'], 'project')
 
     def test_argparse_default_no_token(self):
@@ -650,7 +764,7 @@ class TestConfigArgparse(base.TestCase):
         opts, _remain = parser.parse_known_args(
             ['--os-auth-token', 'very-bad-things',
              '--os-auth-type', 'token'])
-        cc = c.get_one_cloud(argparse=opts)
+        cc = c.get_one_cloud(argparse=opts, validate=False)
         self.assertEqual(cc.config['auth_type'], 'token')
         self.assertEqual(cc.config['auth']['token'], 'very-bad-things')
 
@@ -670,6 +784,17 @@ class TestConfigArgparse(base.TestCase):
         self.assertEqual(cc.config['auth']['password'], 'pass')
         self.assertEqual(cc.config['auth']['auth_url'], 'auth-url')
 
+    def test_argparse_action_append_no_underscore(self):
+        c = config.OpenStackConfig(config_files=[self.no_yaml],
+                                   vendor_files=[self.no_yaml],
+                                   secure_files=[self.no_yaml])
+        parser = argparse.ArgumentParser()
+        parser.add_argument('--foo', action='append')
+        argv = ['--foo', '1', '--foo', '2']
+        c.register_argparse_arguments(parser, argv=argv)
+        opts, _remain = parser.parse_known_args(argv)
+        self.assertEqual(opts.foo, ['1', '2'])
+
     def test_argparse_underscores_duplicate(self):
         c = config.OpenStackConfig(config_files=[self.no_yaml],
                                    vendor_files=[self.no_yaml],
@@ -735,7 +860,7 @@ class TestConfigArgparse(base.TestCase):
         self.assertEqual(opts.http_timeout, '20')
         with testtools.ExpectedException(AttributeError):
             opts.os_network_service_type
-        cloud = c.get_one_cloud(argparse=opts, verify=False)
+        cloud = c.get_one_cloud(argparse=opts, validate=False)
         self.assertEqual(cloud.config['service_type'], 'network')
         self.assertEqual(cloud.config['interface'], 'admin')
         self.assertEqual(cloud.config['api_timeout'], '20')
@@ -756,7 +881,7 @@ class TestConfigArgparse(base.TestCase):
         self.assertIsNone(opts.os_network_service_type)
         self.assertIsNone(opts.os_network_api_version)
         self.assertEqual(opts.network_api_version, '4')
-        cloud = c.get_one_cloud(argparse=opts, verify=False)
+        cloud = c.get_one_cloud(argparse=opts, validate=False)
         self.assertEqual(cloud.config['service_type'], 'network')
         self.assertEqual(cloud.config['interface'], 'admin')
         self.assertEqual(cloud.config['network_api_version'], '4')
@@ -783,7 +908,7 @@ class TestConfigArgparse(base.TestCase):
         self.assertEqual(opts.os_endpoint_type, 'admin')
         self.assertIsNone(opts.os_network_api_version)
         self.assertEqual(opts.network_api_version, '4')
-        cloud = c.get_one_cloud(argparse=opts, verify=False)
+        cloud = c.get_one_cloud(argparse=opts, validate=False)
         self.assertEqual(cloud.config['service_type'], 'compute')
         self.assertEqual(cloud.config['network_service_type'], 'badtype')
         self.assertEqual(cloud.config['interface'], 'admin')
@@ -843,13 +968,6 @@ class TestConfigDefault(base.TestCase):
         self._assert_cloud_details(cc)
         self.assertEqual('password', cc.auth_type)
 
-    def test_set_default_before_init(self):
-        config.set_default('identity_api_version', '4')
-        c = config.OpenStackConfig(config_files=[self.cloud_yaml],
-                                   vendor_files=[self.vendor_yaml])
-        cc = c.get_one_cloud(cloud='_test-cloud_', argparse=None)
-        self.assertEqual('4', cc.identity_api_version)
-
 
 class TestBackwardsCompatibility(base.TestCase):
 
@@ -957,6 +1075,7 @@ class TestBackwardsCompatibility(base.TestCase):
             'networks': [
                 {'name': 'private', 'routes_externally': False,
                  'nat_destination': False, 'default_interface': False,
+                 'nat_source': False,
                  'routes_ipv4_externally': False,
                  'routes_ipv6_externally': False},
             ]
